@@ -1,4 +1,4 @@
-/* $Id: browser.h,v 3.3 2005/09/17 20:46:10 brendan Exp $ */
+/* $Id$ */
 /*
  * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>
  * 
@@ -55,4 +55,10 @@ struct browser_state
 #endif
 };
 
+struct mailbox_state
+{
+  unsigned int new;
+  unsigned int old;
+  unsigned int messages;
+};
 #endif /* _BROWSER_H */
