@@ -18,13 +18,11 @@
  *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-/* this comment bumps Id because of the attachment counting patch. */
-
 #if HAVE_CONFIG_H
 #include "config.h"
 #endif				/* HAVE_CONFIG_H */
 
-#if HAVE_QDBM
+#if HAVE_VILLA_H
 #include <depot.h>
 #include <cabin.h>
 #include <villa.h>
@@ -41,6 +39,7 @@
 #endif
 #include "mutt.h"
 #include "hcache.h"
+#include "hcversion.h"
 #ifdef USE_IMAP
 #include "message.h"
 #endif
@@ -68,6 +67,7 @@ static struct header_cache
 {
   DB_ENV *env;
   DB *db;
+  char *folder;
   unsigned int crc;
   int fd;
   char lockfile[_POSIX_PATH_MAX];
@@ -297,8 +297,10 @@ restore_buffer(BUFFER ** b, const unsigned char *d, int *off)
   restore_char(&(*b)->data, d, off);
   restore_int(&offset, d, off);
   (*b)->dptr = (*b)->data + offset;
-  restore_int((unsigned int *) &(*b)->dsize, d, off);
-  restore_int((unsigned int *) &(*b)->destroy, d, off);
+  restore_int (&used, d, off);
+  (*b)->dsize = used;
+  restore_int (&used, d, off);
+  (*b)->destroy = used;
 }
 
 static unsigned char *
@@ -447,75 +449,8 @@ restore_envelope(ENVELOPE * e, const unsigned char *d, int *off)
   restore_list(&e->userhdrs, d, off);
 }
 
-static unsigned int
-crc32(unsigned int crc, unsigned char const *p, size_t len)
-{
-  int i;
-  while (len--)
-  {
-    crc ^= *p++;
-    for (i = 0; i < 8; i++)
-      crc = (crc >> 1) ^ ((crc & 1) ? 0xedb88320 : 0);
-  }
-  return crc;
-}
-
 static int
-generate_crc32()
-{
-  int crc = 0;
-  SPAM_LIST *sp = SpamList;
-  RX_LIST *rx = NoSpamList;
-
-  crc = crc32(crc, (unsigned char const *) "$Id: hcache.c,v 3.18 2006/10/03 16:40:56 brendan Exp $", mutt_strlen("$Id: hcache.c,v 3.18 2006/10/03 16:40:56 brendan Exp $"));
-
-#if HAVE_LANGINFO_CODESET
-  crc = crc32(crc, (unsigned char const *) Charset, mutt_strlen(Charset));
-  crc = crc32(crc, (unsigned char const *) "HAVE_LANGINFO_CODESET",
-	mutt_strlen("HAVE_LANGINFO_CODESET"));
-#endif
-
-#if EXACT_ADDRESS
-  crc = crc32(crc, (unsigned char const *) "EXACT_ADDRESS",
-	mutt_strlen("EXACT_ADDRESS"));
-#endif
-
-#ifdef USE_POP
-  crc = crc32(crc, (unsigned char const *) "USE_POP", mutt_strlen("USE_POP"));
-#endif
-
-#ifdef MIXMASTER
-  crc = crc32(crc, (unsigned char const *) "MIXMASTER",
-        mutt_strlen("MIXMASTER"));
-#endif
-
-#ifdef USE_IMAP
-  crc = crc32(crc, (unsigned char const *) "USE_IMAP", mutt_strlen("USE_IMAP"));
-  crc = crc32(crc, (unsigned char const *) ImapHeaders,
-        mutt_strlen(ImapHeaders));
-#endif
-  while (sp)
-  {
-    crc = crc32(crc, (unsigned char const *) sp->rx->pattern,
-	  mutt_strlen(sp->rx->pattern));
-    sp = sp->next;
-  }
-
-  crc = crc32(crc, (unsigned char const *) "SPAM_SEPERATOR",
-	mutt_strlen("SPAM_SEPERATOR"));
-
-  while (rx)
-  {
-    crc = crc32(crc, (unsigned char const *) rx->rx->pattern,
-	  mutt_strlen(rx->rx->pattern));
-    rx = rx->next;
-  }
-
-  return crc;
-}
-
-static int
-crc32_matches(const char *d, unsigned int crc)
+crc_matches(const char *d, unsigned int crc)
 {
   int off = sizeof (validate);
   unsigned int mycrc = 0;
@@ -530,37 +465,67 @@ crc32_matches(const char *d, unsigned int crc)
 
 /* Append md5sumed folder to path if path is a directory. */
 static const char *
-mutt_hcache_per_folder(const char *path, const char *folder)
+mutt_hcache_per_folder(const char *path, const char *folder,
+                       hcache_namer_t namer)
 {
-  static char mutt_hcache_per_folder_path[_POSIX_PATH_MAX];
-  struct stat path_stat;
+  static char hcpath[_POSIX_PATH_MAX];
+  struct stat sb;
   MD5_CTX md5;
   unsigned char md5sum[16];
-  int ret;
+  char* s;
+  int ret, plen;
 
-  ret = stat(path, &path_stat);
-  if (ret < 0)
+  plen = mutt_strlen (path);
+
+  ret = stat(path, &sb);
+  if (ret < 0 && path[plen-1] != '/')
     return path;
 
-  if (!S_ISDIR(path_stat.st_mode))
+  if (ret >= 0 && !S_ISDIR(sb.st_mode))
     return path;
 
-  MD5Init(&md5);
-  MD5Update(&md5, (unsigned char *) folder, strlen(folder));
-  MD5Final(md5sum, &md5);
+  if (namer)
+  {
+    snprintf (hcpath, sizeof (hcpath), "%s%s", path,
+              path[plen-1] == '/' ? "" : "/");
+    if (path[plen-1] != '/')
+      plen++;
 
-  ret = snprintf(mutt_hcache_per_folder_path, _POSIX_PATH_MAX,
-		 "%s/%02x%02x%02x%02x%02x%02x%02x%02x"
-		 "%02x%02x%02x%02x%02x%02x%02x%02x",
-		 path, md5sum[0], md5sum[1], md5sum[2], md5sum[3],
-		 md5sum[4], md5sum[5], md5sum[6], md5sum[7], md5sum[8],
-		 md5sum[9], md5sum[10], md5sum[11], md5sum[12],
-		 md5sum[13], md5sum[14], md5sum[15]);
+    ret = namer (folder, hcpath + plen, sizeof (hcpath) - plen);
+  }
+  else
+  {
+    MD5Init(&md5);
+    MD5Update(&md5, (unsigned char *) folder, strlen(folder));
+    MD5Final(md5sum, &md5);
 
+    ret = snprintf(hcpath, _POSIX_PATH_MAX,
+                   "%s/%02x%02x%02x%02x%02x%02x%02x%02x"
+                   "%02x%02x%02x%02x%02x%02x%02x%02x",
+                   path, md5sum[0], md5sum[1], md5sum[2], md5sum[3],
+                   md5sum[4], md5sum[5], md5sum[6], md5sum[7], md5sum[8],
+                   md5sum[9], md5sum[10], md5sum[11], md5sum[12],
+                   md5sum[13], md5sum[14], md5sum[15]);
+  }
+  
   if (ret <= 0)
     return path;
 
-  return mutt_hcache_per_folder_path;
+  if (stat (hcpath, &sb) >= 0)
+    return hcpath;
+
+  s = strchr (hcpath + 1, '/');
+  while (s)
+  {
+    /* create missing path components */
+    *s = '\0';
+    if (stat (hcpath, &sb) < 0 && (errno != ENOENT || mkdir (hcpath, 0777) < 0))
+      return path;
+    *s = '/';
+    s = strchr (s + 1, '/');
+  }
+
+  return hcpath;
 }
 
 /* This function transforms a header into a char so that it is useable by
@@ -570,6 +535,7 @@ mutt_hcache_dump(header_cache_t *h, HEADER * header, int *off,
 		 unsigned long uid_validity)
 {
   unsigned char *d = NULL;
+  HEADER nh;
   *off = 0;
 
   d = lazy_malloc(sizeof (validate));
@@ -587,7 +553,31 @@ mutt_hcache_dump(header_cache_t *h, HEADER * header, int *off,
   d = dump_int(h->crc, d, off);
 
   lazy_realloc(&d, *off + sizeof (HEADER));
-  memcpy(d + *off, header, sizeof (HEADER));
+  memcpy(&nh, header, sizeof (HEADER));
+
+  /* some fields are not safe to cache */
+  nh.tagged = 0;
+  nh.changed = 0;
+  nh.threaded = 0;
+  nh.recip_valid = 0;
+  nh.searched = 0;
+  nh.matched = 0;
+  nh.collapsed = 0;
+  nh.limited = 0;
+  nh.num_hidden = 0;
+  nh.recipient = 0;
+  nh.attach_valid = 0;
+  nh.path = NULL;
+  nh.tree = NULL;
+  nh.thread = NULL;
+#ifdef MIXMASTER
+  nh.chain = NULL;
+#endif
+#if defined USE_POP || defined USE_IMAP
+  nh.data = NULL;
+#endif
+
+  memcpy(d + *off, &nh, sizeof (HEADER));
   *off += sizeof (HEADER);
 
   d = dump_envelope(header->env, d, off);
@@ -639,7 +629,7 @@ mutt_hcache_fetch(header_cache_t *h, const char *filename,
 
   data = mutt_hcache_fetch_raw (h, filename, keylen);
 
-  if (!data || !crc32_matches(data, h->crc))
+  if (!data || !crc_matches(data, h->crc))
   {
     FREE(&data);
     return NULL;
@@ -670,7 +660,8 @@ mutt_hcache_fetch_raw (header_cache_t *h, const char *filename,
     return NULL;
   
 #ifdef HAVE_DB4
-  filename++;			/* skip '/' */
+  if (filename[0] == '/')
+    filename++;
 
   mutt_hcache_dbt_init(&key, (void *) filename, keylen(filename));
   mutt_hcache_dbt_empty_init(&data);
@@ -739,7 +730,8 @@ mutt_hcache_store_raw (header_cache_t* h, const char* filename, void* data,
     return -1;
 
 #if HAVE_DB4
-  filename++;			/* skip '/' */
+  if (filename[0] == '/')
+    filename++;
   
   mutt_hcache_dbt_init(&key, (void *) filename, keylen(filename));
   
@@ -770,48 +762,36 @@ mutt_hcache_store_raw (header_cache_t* h, const char* filename, void* data,
 }
 
 static char* get_foldername(const char *folder) {
-  size_t flen = mutt_strlen (folder);
-  char* p = safe_strdup(folder);
+  char *p = NULL;
+  struct stat st;
 
-  if (flen > 0 && folder[flen-1] == '/')
-    p[flen-1] = '\0';
+  /* if the folder is local, canonify the path to avoid
+   * to ensure equivalent paths share the hcache */
+  if (stat (folder, &st) == 0)
+  {
+    p = safe_malloc (_POSIX_PATH_MAX+1);
+    if (!realpath (folder, p))
+      mutt_str_replace (&p, folder);
+  } else
+    p = safe_strdup (folder);
 
   return p;
 }
 
 #if HAVE_QDBM
-header_cache_t *
-mutt_hcache_open(const char *path, const char *folder)
+static int
+hcache_open_qdbm (struct header_cache* h, const char* path)
 {
-  struct header_cache *h = safe_calloc(1, sizeof (HEADER_CACHE));
   int    flags = VL_OWRITER | VL_OCREAT;
 
-  h->db = NULL;
-  h->folder = get_foldername(folder);
-  h->crc = generate_crc32();
-
-  if (!path || path[0] == '\0')
-  {
-    FREE(&h->folder);
-    FREE(&h);
-    return NULL;
-  }
-
-  path = mutt_hcache_per_folder(path, h->folder);
-
   if (option(OPTHCACHECOMPRESS))
     flags |= VL_OZCOMP;
 
-  h->db = vlopen(path, flags, VL_CMPLEX);
+  h->db = vlopen (path, flags, VL_CMPLEX);
   if (h->db)
-    return h;
+    return 0;
   else
-  {
-    FREE(&h->folder);
-    FREE(&h);
-
-    return NULL;
-  }
+    return -1;
 }
 
 void
@@ -844,41 +824,21 @@ mutt_hcache_delete(header_cache_t *h, const char *filename,
 }
 
 #elif HAVE_GDBM
-
-header_cache_t *
-mutt_hcache_open(const char *path, const char *folder)
+static int
+hcache_open_gdbm (struct header_cache* h, const char* path)
 {
-  struct header_cache *h = safe_calloc(1, sizeof (HEADER_CACHE));
   int pagesize = atoi(HeaderCachePageSize) ? atoi(HeaderCachePageSize) : 16384;
 
-  h->db = NULL;
-  h->folder = get_foldername(folder);
-  h->crc = generate_crc32();
-
-  if (!path || path[0] == '\0')
-  {
-    FREE(&h->folder);
-    FREE(&h);
-    return NULL;
-  }
-
-  path = mutt_hcache_per_folder(path, h->folder);
-
   h->db = gdbm_open((char *) path, pagesize, GDBM_WRCREAT, 00600, NULL);
   if (h->db)
-    return h;
+    return 0;
 
   /* if rw failed try ro */
   h->db = gdbm_open((char *) path, pagesize, GDBM_READER, 00600, NULL);
   if (h->db)
-    return h;
-  else
-  {
-    FREE(&h->folder);
-    FREE(&h);
+    return 0;
 
-    return NULL;
-  }
+  return -1;
 }
 
 void
@@ -929,66 +889,35 @@ mutt_hcache_dbt_empty_init(DBT * dbt)
   dbt->flags = 0;
 }
 
-header_cache_t *
-mutt_hcache_open(const char *path, const char *folder)
+static int
+hcache_open_db4 (struct header_cache* h, const char* path)
 {
   struct stat sb;
-  u_int32_t createflags = DB_CREATE;
   int ret;
-  struct header_cache *h = calloc(1, sizeof (HEADER_CACHE));
-  int pagesize = atoi(HeaderCachePageSize);
-  char* tmp;
+  u_int32_t createflags = DB_CREATE;
+  int pagesize = atoi (HeaderCachePageSize);
 
-  h->crc = generate_crc32();
+  snprintf (h->lockfile, _POSIX_PATH_MAX, "%s-lock-hack", path);
 
-  if (!path || path[0] == '\0')
-  {
-    FREE(&h);
-    return NULL;
-  }
-
-  tmp = get_foldername (folder);
-  path = mutt_hcache_per_folder(path, tmp);
-  snprintf(h->lockfile, _POSIX_PATH_MAX, "%s-lock-hack", path);
-  FREE(&tmp);
-
-  h->fd = open(h->lockfile, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR);
+  h->fd = open (h->lockfile, O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR);
   if (h->fd < 0)
-  {
-    FREE(&h);
-    return NULL;
-  }
+    return -1;
 
-  if (mx_lock_file(h->lockfile, h->fd, 1, 0, 5))
-  {
-    close(h->fd);
-    FREE(&h);
-    return NULL;
-  }
+  if (mx_lock_file (h->lockfile, h->fd, 1, 0, 5))
+    goto fail_close;
 
-  ret = db_env_create(&h->env, 0);
+  ret = db_env_create (&h->env, 0);
   if (ret)
-  {
-    mx_unlock_file(h->lockfile, h->fd, 0);
-    close(h->fd);
-    FREE(&h);
-    return NULL;
-  }
+    goto fail_unlock;
 
   ret = (*h->env->open)(h->env, NULL, DB_INIT_MPOOL | DB_CREATE | DB_PRIVATE,
 	0600);
-  if (!ret)
-  {
-    ret = db_create(&h->db, h->env, 0);
-    if (ret)
-    {
-      h->env->close(h->env, 0);
-      mx_unlock_file(h->lockfile, h->fd, 0);
-      close(h->fd);
-      FREE(&h);
-      return NULL;
-    }
-  }
+  if (ret)
+    goto fail_env;
+
+  ret = db_create (&h->db, h->env, 0);
+  if (ret)
+    goto fail_env;
 
   if (stat(path, &sb) != 0 && errno == ENOENT)
   {
@@ -996,18 +925,24 @@ mutt_hcache_open(const char *path, const char *folder)
     h->db->set_pagesize(h->db, pagesize);
   }
 
-  ret = (*h->db->open)(h->db, NULL, path, folder, DB_BTREE, createflags, 0600);
+  ret = (*h->db->open)(h->db, NULL, path, h->folder, DB_BTREE, createflags,
+                       0600);
   if (ret)
-  {
-    h->db->close(h->db, 0);
-    h->env->close(h->env, 0);
-    mx_unlock_file(h->lockfile, h->fd, 0);
-    close(h->fd);
-    FREE(&h);
-    return NULL;
-  }
+    goto fail_db;
 
-  return h;
+  return 0;
+
+  fail_db:
+  h->db->close (h->db, 0);
+  fail_env:
+  h->env->close (h->env, 0);
+  fail_unlock:
+  mx_unlock_file (h->lockfile, h->fd, 0);
+  fail_close:
+  close (h->fd);
+  unlink (h->lockfile);
+
+  return -1;
 }
 
 void
@@ -1016,11 +951,13 @@ mutt_hcache_close(header_cache_t *h)
   if (!h)
     return;
 
-  h->db->close(h->db, 0);
-  h->env->close(h->env, 0);
-  mx_unlock_file(h->lockfile, h->fd, 0);
-  close(h->fd);
-  FREE(&h);
+  h->db->close (h->db, 0);
+  h->env->close (h->env, 0);
+  mx_unlock_file (h->lockfile, h->fd, 0);
+  close (h->fd);
+  unlink (h->lockfile);
+  FREE (&h->folder);
+  FREE (&h);
 }
 
 int
@@ -1032,9 +969,55 @@ mutt_hcache_delete(header_cache_t *h, const char *filename,
   if (!h)
     return -1;
 
-  filename++;			/* skip '/' */
+  if (filename[0] == '/')
+    filename++;
 
   mutt_hcache_dbt_init(&key, (void *) filename, keylen(filename));
   return h->db->del(h->db, NULL, &key, 0);
 }
 #endif
+
+header_cache_t *
+mutt_hcache_open(const char *path, const char *folder, hcache_namer_t namer)
+{
+  struct header_cache *h = safe_calloc(1, sizeof (HEADER_CACHE));
+  int (*hcache_open) (struct header_cache* h, const char* path);
+  struct stat sb;
+
+#if HAVE_QDBM
+  hcache_open = hcache_open_qdbm;
+#elif HAVE_GDBM
+  hcache_open = hcache_open_gdbm;
+#elif HAVE_DB4
+  hcache_open = hcache_open_db4;
+#endif
+
+  h->db = NULL;
+  h->folder = get_foldername(folder);
+  h->crc = HCACHEVER;
+
+  if (!path || path[0] == '\0')
+  {
+    FREE(&h->folder);
+    FREE(&h);
+    return NULL;
+  }
+
+  path = mutt_hcache_per_folder(path, h->folder, namer);
+
+  if (!hcache_open (h, path))
+    return h;
+  else
+  {
+    /* remove a possibly incompatible version */
+    if (!stat (path, &sb) && !unlink (path))
+    {
+      if (!hcache_open (h, path))
+        return h;
+    }
+    FREE(&h->folder);
+    FREE(&h);
+
+    return NULL;
+  }
+}
