@@ -24,7 +24,10 @@
 struct header_cache;
 typedef struct header_cache header_cache_t;
 
-header_cache_t *mutt_hcache_open(const char *path, const char *folder);
+typedef int (*hcache_namer_t)(const char* path, char* dest, size_t dlen);
+
+header_cache_t *mutt_hcache_open(const char *path, const char *folder,
+  hcache_namer_t namer);
 void mutt_hcache_close(header_cache_t *h);
 HEADER *mutt_hcache_restore(const unsigned char *d, HEADER **oh);
 void *mutt_hcache_fetch(header_cache_t *h, const char *filename, size_t (*keylen)(const char *fn));
