@@ -30,6 +30,9 @@
 #include "message.h"
 #include "mx.h"
 #include "buffy.h"
+#if USE_HCACHE
+#include "hcache.h"
+#endif
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -267,7 +270,7 @@ int imap_exec (IMAP_DATA* idata, const char* cmdstr, int flags)
 
   if (rc != IMAP_CMD_OK)
   {
-    if (flags & IMAP_CMD_FAIL_OK)
+    if (flags & IMAP_CMD_FAIL_OK && idata->status != IMAP_FATAL)
       return -2;
 
     dprint (1, (debugfile, "imap_exec: command failed: %s\n", idata->buf));
@@ -375,11 +378,7 @@ static void cmd_handle_fatal (IMAP_DATA* idata)
   }
 
   if (idata->state < IMAP_SELECTED)
-  {
-    idata->state = IMAP_DISCONNECTED;
-    mutt_socket_close (idata->conn);
-    idata->status = 0;
-  }
+    imap_close_connection (idata);
 }
 
 /* cmd_handle_untagged: fallback parser for otherwise unhandled messages. */
@@ -746,15 +745,33 @@ static void cmd_parse_myrights (IMAP_DATA* idata, const char* s)
       case 'p':
 	mutt_bit_set (idata->ctx->rights, M_ACL_POST);
 	break;
-      case 'c':
-	mutt_bit_set (idata->ctx->rights, M_ACL_CREATE);
-	break;
-      case 'd':
-	mutt_bit_set (idata->ctx->rights, M_ACL_DELETE);
-	break;
       case 'a':
 	mutt_bit_set (idata->ctx->rights, M_ACL_ADMIN);
 	break;
+      case 'k':
+	mutt_bit_set (idata->ctx->rights, M_ACL_CREATE);
+        break;
+      case 'x':
+        mutt_bit_set (idata->ctx->rights, M_ACL_DELMX);
+        break;
+      case 't':
+	mutt_bit_set (idata->ctx->rights, M_ACL_DELETE);
+        break;
+      case 'e':
+        mutt_bit_set (idata->ctx->rights, M_ACL_EXPUNGE);
+        break;
+
+        /* obsolete rights */
+      case 'c':
+	mutt_bit_set (idata->ctx->rights, M_ACL_CREATE);
+        mutt_bit_set (idata->ctx->rights, M_ACL_DELMX);
+	break;
+      case 'd':
+	mutt_bit_set (idata->ctx->rights, M_ACL_DELETE);
+        mutt_bit_set (idata->ctx->rights, M_ACL_EXPUNGE);
+	break;
+      default:
+        dprint(1, (debugfile, "Unknown right: %c\n", *s));
     }
     s++;
   }
@@ -802,23 +819,15 @@ static void cmd_parse_status (IMAP_DATA* idata, char* s)
   BUFFY* inc;
   IMAP_MBOX mx;
   int count;
-  IMAP_STATUS *status, sb;
-  int olduv, oldun;
+  IMAP_STATUS *status;
+  unsigned int olduv, oldun;
 
   mailbox = imap_next_word (s);
   s = imap_next_word (mailbox);
   *(s - 1) = '\0';
   imap_unmunge_mbox_name (mailbox);
 
-  if (!(status = imap_mboxcache_get (idata, mailbox)))
-  {
-    /* ugly interface - why should I look up what I just added? */
-    memset (&sb, 0, sizeof (IMAP_STATUS));
-    sb.name = mailbox;
-    idata->mboxcache = mutt_add_list_n (idata->mboxcache, &sb, sizeof (IMAP_STATUS));
-    status = imap_mboxcache_get (idata, mailbox);
-    status->name = safe_strdup (mailbox);
-  }
+  status = imap_mboxcache_get (idata, mailbox, 1);
   olduv = status->uidvalidity;
   oldun = status->uidnext;
 
@@ -847,7 +856,7 @@ static void cmd_parse_status (IMAP_DATA* idata, char* s)
     if (*s && *s != ')')
       s = imap_next_word (s);
   }
-  dprint (2, (debugfile, "%s (UIDVALIDITY: %d, UIDNEXT: %d) %d messages, %d recent, %d unseen\n",
+  dprint (3, (debugfile, "%s (UIDVALIDITY: %d, UIDNEXT: %d) %d messages, %d recent, %d unseen\n",
               status->name, status->uidvalidity, status->uidnext,
               status->messages, status->recent, status->unseen));
 
@@ -858,7 +867,7 @@ static void cmd_parse_status (IMAP_DATA* idata, char* s)
     return;
   }
 
-  dprint (2, (debugfile, "Running default STATUS handler\n"));
+  dprint (3, (debugfile, "Running default STATUS handler\n"));
 
   /* should perhaps move this code back to imap_buffy_check */
   for (inc = Incoming; inc; inc = inc->next)
@@ -886,21 +895,24 @@ static void cmd_parse_status (IMAP_DATA* idata, char* s)
 
       if (value && !imap_mxcmp (mailbox, value))
       {
-        dprint (2, (debugfile, "Found %s in buffy list (OV: %d ON: %d U: %d)\n",
+        dprint (3, (debugfile, "Found %s in buffy list (OV: %d ON: %d U: %d)\n",
                     mailbox, olduv, oldun, status->unseen));
         
         if (olduv && olduv == status->uidvalidity)
         {
           if (oldun < status->uidnext)
-          {
             inc->new = status->unseen;
-          }
         }
-        else
+        else if (!olduv && !oldun)
+	  /* first check per session, use recent. might need a flag for this. */
+	  inc->new = status->recent;
+	else
           inc->new = status->unseen;
 
-        /* forced back to keep detecting new mail until the mailbox is opened */
-        status->uidnext = oldun;
+	if (inc->new)
+	  /* force back to keep detecting new mail until the mailbox is
+	     opened */
+	  status->uidnext = oldun;
 
         FREE (&value);
         return;
