@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@mutt.org>
  * Copyright (C) 1996-9 Brandon Long <blong@fiction.net>
- * Copyright (C) 1999-2006 Brendan Cully <brendan@kublai.com>
+ * Copyright (C) 1999-2007 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -63,6 +63,7 @@ int imap_access (const char* path, int flags)
   char buf[LONG_STRING];
   char mailbox[LONG_STRING];
   char mbox[LONG_STRING];
+  int rc;
 
   if (imap_parse_path (path, &mx))
     return -1;
@@ -82,8 +83,14 @@ int imap_access (const char* path, int flags)
     FREE (&mx.mbox);
     return 0;
   }
-
   FREE (&mx.mbox);
+
+  if (imap_mboxcache_get (idata, mailbox, 0))
+  {
+    dprint (3, (debugfile, "imap_access: found %s in cache\n", mailbox));
+    return 0;
+  }
+
   imap_munge_mbox_name (mbox, sizeof (mbox), mailbox);
 
   if (mutt_bit_isset (idata->capabilities, IMAP4REV1))
@@ -96,10 +103,10 @@ int imap_access (const char* path, int flags)
     return -1;
   }
 
-  if (imap_exec (idata, buf, IMAP_CMD_FAIL_OK) < 0)
+  if ((rc = imap_exec (idata, buf, IMAP_CMD_FAIL_OK)) < 0)
   {
     dprint (1, (debugfile, "imap_access: Can't check STATUS of %s\n", mbox));
-    return -1;
+    return rc;
   }
 
   return 0;
@@ -242,10 +249,7 @@ void imap_expunge_mailbox (IMAP_DATA* idata)
   int i, cacheno;
 
 #if USE_HCACHE
-  header_cache_t *hc;
   char uidbuf[32];
-  
-  hc = mutt_hcache_open (HeaderCache, idata->ctx->path);
 #endif
 
   for (i = 0; i < idata->ctx->msgcount; i++)
@@ -257,13 +261,14 @@ void imap_expunge_mailbox (IMAP_DATA* idata)
       dprint (2, (debugfile, "Expunging message UID %d.\n", HEADER_DATA (h)->uid));
 
       h->active = 0;
+      idata->ctx->size -= h->content->length;
 
       imap_cache_del (idata, h);
 #if USE_HCACHE
-      if (hc)
+      if (idata->hcache)
       {
         sprintf (uidbuf, "/%u", HEADER_DATA(h)->uid);
-        mutt_hcache_delete (hc, uidbuf, imap_hcache_keylen);
+        mutt_hcache_delete (idata->hcache, uidbuf, imap_hcache_keylen);
       }
 #endif
 
@@ -280,10 +285,6 @@ void imap_expunge_mailbox (IMAP_DATA* idata)
     }
   }
 
-#if USE_HCACHE
-  mutt_hcache_close (hc);
-#endif
-
   /* We may be called on to expunge at any time. We can't rely on the caller
    * to always know to rethread */
   mx_update_tables (idata->ctx, 0);
@@ -341,8 +342,12 @@ IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags)
     }
     if (flags & M_IMAP_CONN_NOSELECT && idata && idata->state >= IMAP_SELECTED)
       continue;
+    if (idata && idata->status == IMAP_FATAL)
+      continue;
     break;
   }
+  if (!conn)
+	  return NULL; /* this happens when the initial connection fails */
 
   if (!idata)
   {
@@ -376,6 +381,8 @@ IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags)
   }
   if (new && idata->state == IMAP_AUTHENTICATED)
   {
+    /* capabilities may have changed */
+    imap_cmd_queue (idata, "CAPABILITY");
     /* get root delimiter, '/' as default */
     idata->delim = '/';
     imap_cmd_queue (idata, "LIST \"\" \"\"");
@@ -399,8 +406,7 @@ int imap_open_connection (IMAP_DATA* idata)
 
   if (imap_cmd_step (idata) != IMAP_CMD_CONTINUE)
   {
-    mutt_socket_close (idata->conn);
-    idata->state = IMAP_DISCONNECTED;
+    imap_close_connection (idata);
     return -1;
   }
 
@@ -467,14 +473,21 @@ int imap_open_connection (IMAP_DATA* idata)
 
 #if defined(USE_SSL)
  err_close_conn:
-  mutt_socket_close (idata->conn);
-  idata->state = IMAP_DISCONNECTED;
+  imap_close_connection (idata);
 #endif
  bail:
   FREE (&idata->capstr);
   return -1;
 }
 
+void imap_close_connection(IMAP_DATA* idata)
+{
+  mutt_socket_close (idata->conn);
+  idata->state = IMAP_DISCONNECTED;
+  idata->seqno = idata->nextcmd = idata->lastcmd = idata->status = 0;
+  memset (idata->cmds, 0, sizeof (IMAP_COMMAND) * IMAP_PIPELINE_DEPTH);
+}
+
 /* imap_get_flags: Make a simple list out of a FLAGS response.
  *   return stream following FLAGS response */
 static char* imap_get_flags (LIST** hflags, char* s)
@@ -536,7 +549,7 @@ int imap_open_mailbox (CONTEXT* ctx)
 {
   CONNECTION *conn;
   IMAP_DATA *idata;
-  IMAP_STATUS* status, sb;
+  IMAP_STATUS* status;
   char buf[LONG_STRING];
   char bufout[LONG_STRING];
   int count = 0;
@@ -559,6 +572,7 @@ int imap_open_mailbox (CONTEXT* ctx)
 
   /* once again the context is new */
   ctx->data = idata;
+  ctx->mx_close = imap_close_mailbox;
 
   /* Clean up path and replace the one in the ctx */
   imap_fix_path (idata, mx.mbox, buf, sizeof (buf));
@@ -611,14 +625,8 @@ int imap_open_mailbox (CONTEXT* ctx)
 
   imap_cmd_start (idata, bufout);
 
-  if (!(status = imap_mboxcache_get (idata, idata->mailbox)))
-  {
-    memset (&sb, 0, sizeof (IMAP_STATUS));
-    sb.name = idata->mailbox;
-    idata->mboxcache = mutt_add_list_n (idata->mboxcache, &sb, sizeof (IMAP_STATUS));
-    status = imap_mboxcache_get (idata, idata->mailbox);
-    status->name = safe_strdup (idata->mailbox);
-  }
+  status = imap_mboxcache_get (idata, idata->mailbox, 1);
+
   do
   {
     char *pc;
@@ -734,6 +742,9 @@ int imap_open_mailbox (CONTEXT* ctx)
   ctx->hdrs = safe_calloc (count, sizeof (HEADER *));
   ctx->v2r = safe_calloc (count, sizeof (int));
   ctx->msgcount = 0;
+#ifdef USE_HCACHE
+  idata->hcache = imap_hcache_open (idata, idata->ctx->path);
+#endif
   if (count && (imap_read_headers (idata, 0, count-1) < 0))
   {
     mutt_error _("Error opening mailbox");
@@ -746,6 +757,9 @@ int imap_open_mailbox (CONTEXT* ctx)
   return 0;
 
  fail:
+#ifdef USE_HCACHE
+  mutt_hcache_close (idata->hcache);
+#endif
   if (idata->state == IMAP_SELECTED)
     idata->state = IMAP_AUTHENTICATED;
  fail_noidata:
@@ -760,6 +774,7 @@ int imap_open_mailbox_append (CONTEXT *ctx)
   char buf[LONG_STRING];
   char mailbox[LONG_STRING];
   IMAP_MBOX mx;
+  int rc;
 
   if (imap_parse_path (ctx->path, &mx))
     return -1;
@@ -782,9 +797,12 @@ int imap_open_mailbox_append (CONTEXT *ctx)
   FREE (&mx.mbox);
 
   /* really we should also check for W_OK */
-  if (!imap_access (ctx->path, F_OK))
+  if ((rc = imap_access (ctx->path, F_OK)) == 0)
     return 0;
 
+  if (rc == -1)
+    return -1;
+
   snprintf (buf, sizeof (buf), _("Create %s?"), mailbox);
   if (option (OPTCONFIRMCREATE) && mutt_yesorno (buf, 1) < 1)
     return -1;
@@ -1089,7 +1107,6 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
   int n;
   int rc;
 #if USE_HCACHE
-  void* hc = NULL;
   char uidbuf[32];
 #endif
   
@@ -1136,11 +1153,6 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
     }
   }
 
-#if USE_HCACHE
-  if (expunge && ctx->closing)
-    hc = mutt_hcache_open (HeaderCache, idata->ctx->path);
-#endif
-
   /* save messages with real (non-flag) changes */
   for (n = 0; n < ctx->msgcount; n++)
   {
@@ -1149,10 +1161,10 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
     if (h->deleted)
       imap_cache_del (idata, h);
 #if USE_HCACHE
-    if (hc && h->deleted)
+    if (idata->hcache && h->deleted)
     {
       sprintf (uidbuf, "/%u", HEADER_DATA(h)->uid);
-      mutt_hcache_delete (hc, uidbuf, imap_hcache_keylen);
+      mutt_hcache_delete (idata->hcache, uidbuf, imap_hcache_keylen);
     }
 #endif
     if (h->active && h->changed)
@@ -1252,9 +1264,6 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
 
   rc = 0;
  out:
-#if USE_HCACHE
-  mutt_hcache_close (hc);
-#endif
   if (cmd.data)
     FREE (&cmd.data);
   if (appendctx)
@@ -1266,7 +1275,7 @@ int imap_sync_mailbox (CONTEXT* ctx, int expunge, int* index_hint)
 }
 
 /* imap_close_mailbox: clean up IMAP data in CONTEXT */
-void imap_close_mailbox (CONTEXT* ctx)
+int imap_close_mailbox (CONTEXT* ctx)
 {
   IMAP_DATA* idata;
   int i;
@@ -1274,7 +1283,7 @@ void imap_close_mailbox (CONTEXT* ctx)
   idata = (IMAP_DATA*) ctx->data;
   /* Check to see if the mailbox is actually open */
   if (!idata)
-    return;
+    return 0;
 
   if (ctx == idata->ctx)
   {
@@ -1313,7 +1322,12 @@ void imap_close_mailbox (CONTEXT* ctx)
     }
   }
 
+#ifdef USE_HCACHE
+  mutt_hcache_close (idata->hcache);
+#endif
   mutt_bcache_close (&idata->bcache);
+
+  return 0;
 }
 
 /* use the NOOP or IDLE command to poll for new mail
@@ -1468,7 +1482,8 @@ int imap_buffy_check (int force)
     }
     
     imap_munge_mbox_name (munged, sizeof (munged), name);
-    snprintf (command, sizeof (command), "STATUS %s (UIDNEXT UIDVALIDITY UNSEEN)", munged);
+    snprintf (command, sizeof (command),
+	      "STATUS %s (UIDNEXT UIDVALIDITY UNSEEN RECENT)", munged);
 
     if (imap_cmd_queue (idata, command) < 0)
     {
@@ -1542,17 +1557,25 @@ int imap_status (char* path, int queue)
     imap_exec (idata, buf, 0);
 
   queued = 0;
-  if ((status = imap_mboxcache_get (idata, mbox)))
+  if ((status = imap_mboxcache_get (idata, mbox, 0)))
     return status->messages;
   
   return 0;
 }
 
-/* return cached mailbox stats or NULL */
-IMAP_STATUS* imap_mboxcache_get (IMAP_DATA* idata, const char* mbox)
+/* return cached mailbox stats or NULL if create is 0 */
+IMAP_STATUS* imap_mboxcache_get (IMAP_DATA* idata, const char* mbox, int create)
 {
   LIST* cur;
   IMAP_STATUS* status;
+  IMAP_STATUS scache;
+#ifdef USE_HCACHE
+  header_cache_t *hc = NULL;
+  ciss_url_t url;
+  char urlstr[LONG_STRING];
+  unsigned int *uidvalidity = NULL;
+  unsigned int *uidnext = NULL;
+#endif
   
   for (cur = idata->mboxcache; cur; cur = cur->next)
   {
@@ -1561,8 +1584,48 @@ IMAP_STATUS* imap_mboxcache_get (IMAP_DATA* idata, const char* mbox)
     if (!imap_mxcmp (mbox, status->name))
       return status;
   }
-  
-  return NULL;
+  status = NULL;
+
+  /* lame */
+  if (create)
+  {
+    memset (&scache, 0, sizeof (scache));
+    scache.name = (char*)mbox;
+    idata->mboxcache = mutt_add_list_n (idata->mboxcache, &scache,
+                                        sizeof (scache));
+    status = imap_mboxcache_get (idata, mbox, 0);
+    status->name = safe_strdup (mbox);
+  }
+
+#ifdef USE_HCACHE
+  mutt_account_tourl (&idata->conn->account, &url);
+  url.path = (char*)mbox;
+  url_ciss_tostring (&url, urlstr, sizeof (urlstr), 0);
+  hc = mutt_hcache_open (HeaderCache, urlstr);
+  if (hc)
+  {
+    uidvalidity = mutt_hcache_fetch_raw (hc, "/UIDVALIDITY", imap_hcache_keylen);
+    uidnext = mutt_hcache_fetch_raw (hc, "/UIDNEXT", imap_hcache_keylen);
+    mutt_hcache_close (hc);
+    if (uidvalidity)
+    {
+      if (!status)
+      {
+        FREE (&uidvalidity);
+        FREE (&uidnext);
+        return imap_mboxcache_get (idata, mbox, 1);
+      }
+      status->uidvalidity = *uidvalidity;
+      status->uidnext = uidnext ? *uidnext: 0;
+      dprint (3, (debugfile, "mboxcache: hcache uidvalidity %d, uidnext %d\n",
+                  status->uidvalidity, status->uidnext));
+    }
+    FREE (&uidvalidity);
+    FREE (&uidnext);
+  }
+#endif
+
+  return status;
 }
 
 void imap_mboxcache_free (IMAP_DATA* idata)
@@ -1821,7 +1884,7 @@ imap_complete_hosts (char *dest, size_t len)
     }
   }
   
-  for (conn = mutt_socket_head (); conn && conn->next; conn = conn->next)
+  for (conn = mutt_socket_head (); conn; conn = conn->next)
   {
     ciss_url_t url;
     char urlstr[LONG_STRING];
@@ -1863,7 +1926,7 @@ int imap_complete(char* dest, size_t dlen, char* path) {
   IMAP_MBOX mx;
   int rc;
 
-  if (imap_parse_path (path, &mx) || !mx.mbox)
+  if (imap_parse_path (path, &mx))
   {
     strfcpy (dest, path, dlen);
     return imap_complete_hosts (dest, dlen);
