@@ -24,6 +24,9 @@
 #include "mutt_curses.h"
 #include "mutt_socket.h"
 #include "bcache.h"
+#ifdef USE_HCACHE
+#include "hcache.h"
+#endif
 
 /* -- symbols -- */
 #define IMAP_PORT 143
@@ -124,18 +127,6 @@ typedef struct
   char* path;
 } IMAP_CACHE;
 
-typedef struct 
-{
-  int type;
-  int listable;
-  char *prefix;
-  char delim;
-  int home_namespace;
-  /* We get these when we check if namespace exists - cache them */
-  int noselect;			
-  int noinferiors;
-} IMAP_NAMESPACE_INFO;
-
 typedef struct
 {
   char* name;
@@ -220,6 +211,9 @@ typedef struct
 
   /* all folder flags - system flags AND keywords */
   LIST *flags;
+#ifdef USE_HCACHE
+  header_cache_t *hcache;
+#endif
 } IMAP_DATA;
 /* I wish that were called IMAP_CONTEXT :( */
 
@@ -230,11 +224,13 @@ typedef struct
 /* imap.c */
 int imap_create_mailbox (IMAP_DATA* idata, char* mailbox);
 int imap_rename_mailbox (IMAP_DATA* idata, IMAP_MBOX* mx, const char* newname);
-IMAP_STATUS* imap_mboxcache_get (IMAP_DATA* idata, const char* mbox);
+IMAP_STATUS* imap_mboxcache_get (IMAP_DATA* idata, const char* mbox,
+                                 int create);
 void imap_mboxcache_free (IMAP_DATA* idata);
 int imap_make_msg_set (IMAP_DATA* idata, BUFFER* buf, int flag, int changed,
                        int invert);
 int imap_open_connection (IMAP_DATA* idata);
+void imap_close_connection (IMAP_DATA* idata);
 IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags);
 int imap_read_literal (FILE* fp, IMAP_DATA* idata, long bytes, progress_t*);
 void imap_expunge_mailbox (IMAP_DATA* idata);
@@ -262,6 +258,12 @@ char* imap_set_flags (IMAP_DATA* idata, HEADER* h, char* s);
 int imap_cache_del (IMAP_DATA* idata, HEADER* h);
 
 /* util.c */
+#ifdef USE_HCACHE
+header_cache_t* imap_hcache_open (IMAP_DATA* idata, const char* path);
+HEADER* imap_hcache_get (IMAP_DATA* idata, unsigned int uid);
+int imap_hcache_put (IMAP_DATA* idata, HEADER* h);
+#endif
+
 int imap_continue (const char* msg, const char* resp);
 void imap_error (const char* where, const char* msg);
 IMAP_DATA* imap_new_idata (void);
