@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 1996-9 Brandon Long <blong@fiction.net>
- * Copyright (C) 1999-2006 Brendan Cully <brendan@kublai.com>
+ * Copyright (C) 1999-2007 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -45,6 +45,7 @@
 
 static FILE* msg_cache_get (IMAP_DATA* idata, HEADER* h);
 static FILE* msg_cache_put (IMAP_DATA* idata, HEADER* h);
+static int msg_cache_commit (IMAP_DATA* idata, HEADER* h);
 
 static void flush_buffer(char* buf, size_t* len, CONNECTION* conn);
 static int msg_fetch_header (CONTEXT* ctx, IMAP_HEADER* h, char* buf,
@@ -74,11 +75,9 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   progress_t progress;
 
 #if USE_HCACHE
-  header_cache_t *hc = NULL;
   unsigned int *uid_validity = NULL;
   unsigned int *uidnext = NULL;
   int evalhc = 0;
-  char uid_buf[64];
 #endif /* USE_HCACHE */
 
   ctx = idata->ctx;
@@ -120,16 +119,15 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   idata->newMailCount = 0;
 
 #if USE_HCACHE
-  if (!msgbegin)
-    hc = mutt_hcache_open (HeaderCache, ctx->path);
-
-  if (hc)
+  if (idata->hcache && !msgbegin)
   {
-    uid_validity = mutt_hcache_fetch_raw (hc, "/UIDVALIDITY", imap_hcache_keylen);
-    uidnext = mutt_hcache_fetch_raw (hc, "/UIDNEXT", imap_hcache_keylen);
+    uid_validity = mutt_hcache_fetch_raw (idata->hcache, "/UIDVALIDITY", imap_hcache_keylen);
+    uidnext = mutt_hcache_fetch_raw (idata->hcache, "/UIDNEXT", imap_hcache_keylen);
     if (uid_validity && uidnext && *uid_validity == idata->uid_validity
         && *uidnext > 0)
       evalhc = 1;
+    else
+      FREE (&uidnext);
     FREE (&uid_validity);
   }
   if (evalhc)
@@ -169,12 +167,9 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
 	}
 
         idx = h.sid - 1;
-        sprintf(uid_buf, "/%u", h.data->uid); /* XXX --tg 21:41 04-07-11 */
-        uid_validity = (unsigned int*)mutt_hcache_fetch (hc, uid_buf, &imap_hcache_keylen);
-
-        if (uid_validity != NULL && *uid_validity == idata->uid_validity)
+        ctx->hdrs[idx] = imap_hcache_get (idata, h.data->uid);
+        if (ctx->hdrs[idx])
         {
-  	  ctx->hdrs[idx] = mutt_hcache_restore((unsigned char *) uid_validity, 0);
   	  ctx->hdrs[idx]->index = idx;
   	  /* messages which have not been expunged are ACTIVE (borrowed from mh 
   	   * folders) */
@@ -189,13 +184,12 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
           ctx->hdrs[idx]->data = (void *) (h.data);
 
           ctx->msgcount++;
+          ctx->size += ctx->hdrs[idx]->content->length;
         }
 	else
 	  /* bad header in the cache, we'll have to refetch.
 	   * TODO: consider the possibility of a holey cache. */
           imap_free_header_data((void**) &h.data);
-  
-        FREE(&uid_validity);
       }
       while (rc != IMAP_CMD_OK && mfhrc == -1);
       if (rc == IMAP_CMD_OK)
@@ -205,7 +199,6 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
         if (h.data)
           imap_free_header_data ((void**) &h.data);
         fclose (fp);
-        mutt_hcache_close (hc);
         return -1;
       }
     }
@@ -291,10 +284,10 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
         0, 0);
       /* content built as a side-effect of mutt_read_rfc822_header */
       ctx->hdrs[idx]->content->length = h.content_length;
+      ctx->size += h.content_length;
 
 #if USE_HCACHE
-      sprintf(uid_buf, "/%u", h.data->uid);
-      mutt_hcache_store(hc, uid_buf, ctx->hdrs[idx], idata->uid_validity, &imap_hcache_keylen);
+      imap_hcache_put (idata, ctx->hdrs[idx]);
 #endif /* USE_HCACHE */
 
       ctx->msgcount++;
@@ -307,9 +300,6 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
       if (h.data)
         imap_free_header_data ((void**) &h.data);
       fclose (fp);
-#if USE_HCACHE
-      mutt_hcache_close (hc);
-#endif /* USE_HCACHE */
       return -1;
     }
 
@@ -324,11 +314,11 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
     }
   }
 
-  if (maxuid && (status = imap_mboxcache_get (idata, idata->mailbox)))
+  if (maxuid && (status = imap_mboxcache_get (idata, idata->mailbox, 0)))
   status->uidnext = maxuid + 1;
 
 #if USE_HCACHE
-  mutt_hcache_store_raw (hc, "/UIDVALIDITY", &idata->uid_validity,
+  mutt_hcache_store_raw (idata->hcache, "/UIDVALIDITY", &idata->uid_validity,
                          sizeof (idata->uid_validity), imap_hcache_keylen);
   if (maxuid && idata->uidnext < maxuid + 1)
   {
@@ -336,9 +326,8 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
     idata->uidnext = maxuid + 1;
   }
   if (idata->uidnext > 1)
-    mutt_hcache_store_raw (hc, "/UIDNEXT", &idata->uidnext,
+    mutt_hcache_store_raw (idata->hcache, "/UIDNEXT", &idata->uidnext,
 			   sizeof (idata->uidnext), imap_hcache_keylen);
-  mutt_hcache_close (hc);
 #endif /* USE_HCACHE */
 
   fclose(fp);
@@ -500,7 +489,9 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
   if (!fetched || !imap_code (idata->buf))
     goto bail;
 
-parsemsg:
+  msg_cache_commit (idata, h);
+
+  parsemsg:
   /* Update the header information.  Previously, we only downloaded a
    * portion of the headers, those required for the main display.
    */
@@ -872,8 +863,8 @@ static body_cache_t *msg_cache_open (IMAP_DATA *idata)
     }
     else
       *s = *p;
-    *p++;
-    *s++;
+    p++;
+    s++;
   }
   *s = '\0';
 
@@ -901,7 +892,20 @@ static FILE* msg_cache_put (IMAP_DATA* idata, HEADER* h)
 
   idata->bcache = msg_cache_open (idata);
   snprintf (id, sizeof (id), "%u-%u", idata->uid_validity, HEADER_DATA(h)->uid);
-  return mutt_bcache_put (idata->bcache, id);
+  return mutt_bcache_put (idata->bcache, id, 1);
+}
+
+static int msg_cache_commit (IMAP_DATA* idata, HEADER* h)
+{
+  char id[_POSIX_PATH_MAX];
+
+  if (!idata || !h)
+    return -1;
+
+  idata->bcache = msg_cache_open (idata);
+  snprintf (id, sizeof (id), "%u-%u", idata->uid_validity, HEADER_DATA(h)->uid);
+
+  return mutt_bcache_commit (idata->bcache, id);
 }
 
 int imap_cache_del (IMAP_DATA* idata, HEADER* h)
