@@ -447,9 +447,10 @@ int safe_rename (const char *src, const char *target)
     
     dprint (1, (debugfile, "safe_rename: link (%s, %s) failed: %s (%d)\n", src, target, strerror (errno), errno));
 
-    if (errno == EXDEV)
+    /* FUSE may return ENOSYS. VFAT may return EPERM */
+    if (errno == EXDEV || errno == ENOSYS || errno == EPERM)
     {
-      dprint (1, (debugfile, "safe_rename: errno was EXDEV; trying rename...\n"));
+      dprint (1, (debugfile, "safe_rename: trying rename...\n"));
       if (rename (src, target) == -1) 
       {
 	dprint (1, (debugfile, "safe_rename: rename (%s, %s) failed: %s (%d)\n", src, target, strerror (errno), errno));
@@ -575,18 +576,15 @@ int safe_open (const char *path, int flags)
       rmdir (safe_dir);
       return fd;
     }
-    
+
+    /* NFS and I believe cygwin do not handle movement of open files well */
+    close (fd);
     if (mutt_put_file_in_place (path, safe_file, safe_dir) == -1)
-    {
-      close (fd);
       return -1;
-    }
-  }
-  else
-  {
-    if ((fd = open (path, flags, 0600)) < 0)
-      return fd;
   }
+
+  if ((fd = open (path, flags & ~O_EXCL, 0600)) < 0)
+    return fd;
     
   /* make sure the file is not symlink */
   if (lstat (path, &osb) < 0 || fstat (fd, &nsb) < 0 ||
