@@ -61,7 +61,7 @@ To contact the developers, please mail to <mutt-dev@mutt.org>.\n\
 To report a bug, please visit http://bugs.mutt.org/.\n");
 
 static const char *Notice = N_("\
-Copyright (C) 1996-2006 Michael R. Elkins and others.\n\
+Copyright (C) 1996-2007 Michael R. Elkins and others.\n\
 Mutt comes with ABSOLUTELY NO WARRANTY; for details type `mutt -vv'.\n\
 Mutt is free software, and you are welcome to redistribute it\n\
 under certain conditions; type `mutt -vv' for details.\n");
@@ -69,9 +69,9 @@ under certain conditions; type `mutt -vv' for details.\n");
 static const char *Copyright = N_("\
 Copyright (C) 1996-2004 Michael R. Elkins <me@mutt.org>\n\
 Copyright (C) 1996-2002 Brandon Long <blong@fiction.net>\n\
-Copyright (C) 1997-2006 Thomas Roessler <roessler@does-not-exist.org>\n\
+Copyright (C) 1997-2007 Thomas Roessler <roessler@does-not-exist.org>\n\
 Copyright (C) 1998-2005 Werner Koch <wk@isil.d.shuttle.de>\n\
-Copyright (C) 1999-2006 Brendan Cully <brendan@kublai.com>\n\
+Copyright (C) 1999-2007 Brendan Cully <brendan@kublai.com>\n\
 Copyright (C) 1999-2002 Tommi Komulainen <Tommi.Komulainen@iki.fi>\n\
 Copyright (C) 2000-2002 Edmund Grimley Evans <edmundo@rano.org>\n\
 \n\
@@ -105,12 +105,12 @@ static void mutt_usage (void)
   puts (mutt_make_version ());
 
   puts _(
-"usage: mutt [ -nRyzZ ] [ -e <cmd> ] [ -F <file> ] [ -m <type> ] [ -f <file> ]\n\
-       mutt [ -nR ] [ -e <cmd> ] [ -F <file> ] -Q <query> [ -Q <query> ] [...]\n\
-       mutt [ -nR ] [ -e <cmd> ] [ -F <file> ] -A <alias> [ -A <alias> ] [...]\n\
-       mutt [ -nR ] [ -e <cmd> ] [ -F <file> ] -D\n\
-       mutt [ -nx ] [ -e <cmd> ] [ -a <file> ] [ -F <file> ] [ -H <file> ] [ -i <file> ] [ -s <subj> ] [ -b <addr> ] [ -c <addr> ] <addr> [ ... ]\n\
-       mutt [ -n ] [ -e <cmd> ] [ -F <file> ] -p\n\
+"usage: mutt [<options>] [-z] [-f <file> | -yZ]\n\
+       mutt [<options>] [-x] [-Hi <file>] [-s <subj>] [-bc <addr>] [-a <file> [...]] [--] <addr> [...]\n\
+       mutt [<options>] -p\n\
+       mutt [<options>] -A <alias> [...]\n\
+       mutt [<options>] -Q <query> [...]\n\
+       mutt [<options>] -D\n\
        mutt -v[v]\n");
 
   puts _("\
@@ -143,6 +143,8 @@ options:\n\
   -z\t\texit immediately if there are no messages in the mailbox\n\
   -Z\t\topen the first folder with new message, exit immediately if none\n\
   -h\t\tthis help message");
+  puts _("  --\t\ttreat remaining arguments as addr even if starting with a dash\n\
+\t\twhen using -a with multiple filenames using -- is mandatory");
 
   exit (0);
 }
@@ -167,18 +169,18 @@ static void show_version (void)
   printf (" (%s)", uts.machine);
 
 #ifdef NCURSES_VERSION
-  printf (" [using ncurses %s]", NCURSES_VERSION);
+  printf ("\nncurses: %s (compiled with %s)", curses_version(), NCURSES_VERSION);
 #elif defined(USE_SLANG_CURSES)
-  printf (" [using slang %d]", SLANG_VERSION);
+  printf ("\nslang: %d", SLANG_VERSION);
 #endif
 
 #ifdef _LIBICONV_VERSION
-  printf (" [using libiconv %d.%d]", _LIBICONV_VERSION >> 8,
+  printf ("\nlibiconv: %d.%d", _LIBICONV_VERSION >> 8,
 	  _LIBICONV_VERSION & 0xff);
 #endif
 
 #ifdef HAVE_LIBIDN
-  printf (" [using libidn %s (compiled with %s)]", stringprep_check_version (NULL), 
+  printf ("\nlibidn: %s (compiled with %s)", stringprep_check_version (NULL), 
 	  STRINGPREP_VERSION);
 #endif
   
@@ -257,6 +259,12 @@ static void show_version (void)
         "-USE_IMAP  "
 #endif
 
+#ifdef USE_SMTP
+	"+USE_SMTP  "
+#else
+	"-USE_SMTP  "
+#endif
+
 #ifdef USE_GSS
 	"+USE_GSS  "
 #else
@@ -368,11 +376,6 @@ static void show_version (void)
         );
   
   puts (
-#ifdef BUFFY_SIZE
-	"+BUFFY_SIZE "
-#else
-	"-BUFFY_SIZE "
-#endif
 #ifdef EXACT_ADDRESS
 	"+EXACT_ADDRESS  "
 #else
@@ -539,6 +542,7 @@ int main (int argc, char **argv)
   int dump_variables = 0;
   extern char *optarg;
   extern int optind;
+  int attach_sep = 0;
 
   /* sanity check against stupid administrators */
   
@@ -565,7 +569,14 @@ int main (int argc, char **argv)
 
   memset (Options, 0, sizeof (Options));
   memset (QuadOptions, 0, sizeof (QuadOptions));
-  
+
+  for (i = 1; i < argc; i++)
+    if (!strcmp(argv[i], "--"))
+    {
+      attach_sep = i;
+      break;
+    }
+
   while ((i = getopt (argc, argv, "A:a:b:F:f:c:Dd:e:H:s:i:hm:npQ:RvxyzZ")) != EOF)
     switch (i)
     {
@@ -707,7 +718,11 @@ int main (int argc, char **argv)
   crypt_init ();
 
   if (queries)
+  {
+    for (; optind < argc; optind++)
+      queries = mutt_add_list (queries, argv[optind]);
     return mutt_query_variables (queries);
+  }
   if (dump_variables)
     return mutt_dump_variables();
 
@@ -715,6 +730,8 @@ int main (int argc, char **argv)
   {
     int rv = 0;
     ADDRESS *a;
+    for (; optind < argc; optind++)
+      alias_queries = mutt_add_list (alias_queries, argv[optind]);
     for (; alias_queries; alias_queries = alias_queries->next)
     {
       if ((a = mutt_lookup_alias (alias_queries->data)))
@@ -731,7 +748,12 @@ int main (int argc, char **argv)
     }
     return rv;
   }
-  
+
+  /* if an -a option is present, all non-option arguments before -- are considered attachments */
+  if (attach)
+    for (; optind <= attach_sep; optind++)
+      attach = mutt_add_list (attach, argv[optind]);
+
   if (newMagic)
     mx_set_magic (newMagic);
 
