@@ -71,8 +71,15 @@ void mutt_set_charset (char *charset)
   	|| !ascii_strcasecmp(buffer, "cp932") || !ascii_strcasecmp(buffer, "eucJP-ms"))
   {
     charset_is_ja = 1;
-    charset_to_utf8 = iconv_open ("UTF-8", charset);
-    charset_from_utf8 = iconv_open (charset, "UTF-8");
+
+    /* Note flags=0 to skip charset-hooks: User masters the $charset
+     * name, and we are sure of our "utf-8" constant. So there is no
+     * possibility of wrong name that we would want to try to correct
+     * with a charset-hook. Or rather: If $charset was wrong, we would
+     * want to try to correct... $charset directly.
+     */
+    charset_to_utf8 = mutt_iconv_open ("utf-8", charset, 0);
+    charset_from_utf8 = mutt_iconv_open (charset, "utf-8", 0);
   }
 #endif
 
