@@ -73,12 +73,23 @@ struct mh_sequences
   short *flags;
 };
 
+struct mh_data
+{
+  time_t mtime_cur;
+  mode_t mh_umask;
+};
+
 /* mh_sequences support */
 
 #define MH_SEQ_UNSEEN  (1 << 0)
 #define MH_SEQ_REPLIED (1 << 1)
 #define MH_SEQ_FLAGGED (1 << 2)
 
+static inline struct mh_data *mh_data (CONTEXT *ctx)
+{
+  return (struct mh_data*)ctx->data;
+}
+
 static void mhs_alloc (struct mh_sequences *mhs, int i)
 {
   int j;
@@ -87,9 +98,10 @@ static void mhs_alloc (struct mh_sequences *mhs, int i)
   if (i > mhs->max || !mhs->flags)
   {
     newmax = i + 128;
+    j = mhs->flags ? mhs->max + 1 : 0;
     safe_realloc (&mhs->flags, sizeof (mhs->flags[0]) * (newmax + 1));
-    for (j = mhs->max + 1; j <= newmax; j++)
-      mhs->flags[j] = 0;
+    while (j <= newmax)
+      mhs->flags[j++] = 0;
 
     mhs->max = newmax;
   }
@@ -184,6 +196,23 @@ static void mh_read_sequences (struct mh_sequences *mhs, const char *path)
   safe_fclose (&fp);
 }
 
+static inline mode_t mh_umask (CONTEXT* ctx)
+{
+  struct stat st;
+  struct mh_data* data = mh_data (ctx);
+
+  if (data && data->mh_umask)
+    return data->mh_umask;
+
+  if (stat (ctx->path, &st))
+  {
+    dprint (1, (debugfile, "stat failed on %s\n", ctx->path));
+    return 077;
+  }
+
+  return 0777 & ~st.st_mode;
+}
+
 int mh_buffy (const char *path)
 {
   int i, r = 0;
@@ -202,16 +231,19 @@ static int mh_mkstemp (CONTEXT * dest, FILE ** fp, char **tgt)
 {
   int fd;
   char path[_POSIX_PATH_MAX];
+  mode_t omask;
 
+  omask = umask (mh_umask (dest));
   FOREVER
   {
     snprintf (path, _POSIX_PATH_MAX, "%s/.mutt-%s-%d-%d",
 	      dest->path, NONULL (Hostname), (int) getpid (), Counter++);
-    if ((fd = open (path, O_WRONLY | O_EXCL | O_CREAT, 0600)) == -1)
+    if ((fd = open (path, O_WRONLY | O_EXCL | O_CREAT, 0666)) == -1)
     {
       if (errno != EEXIST)
       {
 	mutt_perror (path);
+	umask (omask);
 	return -1;
       }
     }
@@ -221,6 +253,7 @@ static int mh_mkstemp (CONTEXT * dest, FILE ** fp, char **tgt)
       break;
     }
   }
+  umask (omask);
 
   if ((*fp = fdopen (fd, "w")) == NULL)
   {
@@ -564,19 +597,20 @@ static void maildir_update_mtime (CONTEXT * ctx)
 {
   char buf[_POSIX_PATH_MAX];
   struct stat st;
+  struct mh_data *data = mh_data (ctx);
 
   if (ctx->magic == M_MAILDIR)
   {
     snprintf (buf, sizeof (buf), "%s/%s", ctx->path, "cur");
     if (stat (buf, &st) == 0)
-      ctx->mtime_cur = st.st_mtime;
+      data->mtime_cur = st.st_mtime;
     snprintf (buf, sizeof (buf), "%s/%s", ctx->path, "new");
   }
   else
   {
     snprintf (buf, sizeof (buf), "%s/.mh_sequences", ctx->path);
     if (stat (buf, &st) == 0)
-      ctx->mtime_cur = st.st_mtime;
+      data->mtime_cur = st.st_mtime;
 
     strfcpy (buf, ctx->path, sizeof (buf));
   }
@@ -666,10 +700,19 @@ static int maildir_parse_entry (CONTEXT * ctx, struct maildir ***last,
   {
 #ifdef USE_HCACHE
     if (hc && (data = mutt_hcache_fetch (hc, fname, strlen)))
+    {
       h = mutt_hcache_restore ((unsigned char *) data, NULL);
+      FREE (&data);
+    }
     else
-#endif
+    {
+      h = maildir_parse_message (ctx->magic, buf, is_old, NULL);
+      if (h)
+        mutt_hcache_store (hc, fname, h, 0, strlen);
+    }
+#else
     h = maildir_parse_message (ctx->magic, buf, is_old, NULL);
+#endif
   }
   else
   {
@@ -996,7 +1039,7 @@ void maildir_delayed_parsing (CONTEXT * ctx, struct maildir *md,
     } else
       mutt_free_header (&p->h);
 #if USE_HCACHE
-    FREE(&data);
+    FREE (&data);
 #endif
   }
 #if USE_HCACHE
@@ -1004,6 +1047,13 @@ void maildir_delayed_parsing (CONTEXT * ctx, struct maildir *md,
 #endif
 }
 
+static int mh_close_mailbox (CONTEXT *ctx)
+{
+  FREE (&ctx->data);
+
+  return 0;
+}
+
 /* Read a MH/maildir style mailbox.
  *
  * args:
@@ -1016,6 +1066,7 @@ int mh_read_dir (CONTEXT * ctx, const char *subdir)
   struct maildir *md;
   struct mh_sequences mhs;
   struct maildir **last;
+  struct mh_data *data;
   int count;
   char msgbuf[STRING];
   progress_t progress;
@@ -1024,6 +1075,13 @@ int mh_read_dir (CONTEXT * ctx, const char *subdir)
   snprintf (msgbuf, sizeof (msgbuf), _("Reading %s..."), ctx->path);
   mutt_progress_init (&progress, msgbuf, M_PROGRESS_MSG, ReadInc, 0);
 
+  if (!ctx->data)
+  {
+    ctx->data = safe_calloc(sizeof (struct mh_data), 1);
+    ctx->mx_close = mh_close_mailbox;
+  }
+  data = mh_data (ctx);
+
   maildir_update_mtime (ctx);
 
   md = NULL;
@@ -1047,6 +1105,10 @@ int mh_read_dir (CONTEXT * ctx, const char *subdir)
     maildir_delayed_parsing (ctx, md, &progress);
 
   maildir_move_to_context (ctx, &md);
+
+  if (!data->mh_umask)
+    data->mh_umask = mh_umask (ctx);
+  
   return 0;
 }
 
@@ -1118,6 +1180,7 @@ int maildir_open_new_message (MESSAGE * msg, CONTEXT * dest, HEADER * hdr)
   char path[_POSIX_PATH_MAX];
   char suffix[16];
   char subdir[16];
+  mode_t omask;
 
   if (hdr)
   {
@@ -1136,6 +1199,7 @@ int maildir_open_new_message (MESSAGE * msg, CONTEXT * dest, HEADER * hdr)
   else
     strfcpy (subdir, "new", sizeof (subdir));
 
+  omask = umask (mh_umask (dest));
   FOREVER
   {
     snprintf (path, _POSIX_PATH_MAX, "%s/tmp/%s.%ld.%u_%d.%s%s",
@@ -1145,10 +1209,11 @@ int maildir_open_new_message (MESSAGE * msg, CONTEXT * dest, HEADER * hdr)
     dprint (2, (debugfile, "maildir_open_new_message (): Trying %s.\n",
 		path));
 
-    if ((fd = open (path, O_WRONLY | O_EXCL | O_CREAT, 0600)) == -1)
+    if ((fd = open (path, O_WRONLY | O_EXCL | O_CREAT, 0666)) == -1)
     {
       if (errno != EEXIST)
       {
+	umask (omask);
 	mutt_perror (path);
 	return -1;
       }
@@ -1160,6 +1225,7 @@ int maildir_open_new_message (MESSAGE * msg, CONTEXT * dest, HEADER * hdr)
       break;
     }
   }
+  umask (omask);
 
   if ((msg->fp = fdopen (fd, "w")) == NULL)
   {
@@ -1709,6 +1775,7 @@ int maildir_check_mailbox (CONTEXT * ctx, int *index_hint)
   int i;
   HASH *fnames;			/* hash table for quickly looking up the base filename
 				   for a maildir message */
+  struct mh_data *data = mh_data (ctx);
 
   /* XXX seems like this check belongs in mx_check_mailbox()
    * rather than here.
@@ -1727,14 +1794,14 @@ int maildir_check_mailbox (CONTEXT * ctx, int *index_hint)
   /* determine which subdirectories need to be scanned */
   if (st_new.st_mtime > ctx->mtime)
     changed = 1;
-  if (st_cur.st_mtime > ctx->mtime_cur)
+  if (st_cur.st_mtime > data->mtime_cur)
     changed |= 2;
 
   if (!changed)
     return 0;			/* nothing to do */
 
   /* update the modification times on the mailbox */
-  ctx->mtime_cur = st_cur.st_mtime;
+  data->mtime_cur = st_cur.st_mtime;
   ctx->mtime = st_new.st_mtime;
 
   /* do a fast scan of just the filenames in
@@ -1850,6 +1917,7 @@ int mh_check_mailbox (CONTEXT * ctx, int *index_hint)
   struct mh_sequences mhs;
   HASH *fnames;
   int i;
+  struct mh_data *data = mh_data (ctx);
 
   if (!option (OPTCHECKNEW))
     return 0;
@@ -1877,13 +1945,13 @@ int mh_check_mailbox (CONTEXT * ctx, int *index_hint)
   if (i == -1 && stat (buf, &st_cur) == -1)
     modified = 1;
 
-  if (st.st_mtime > ctx->mtime || st_cur.st_mtime > ctx->mtime_cur)
+  if (st.st_mtime > ctx->mtime || st_cur.st_mtime > data->mtime_cur)
     modified = 1;
 
   if (!modified)
     return 0;
 
-  ctx->mtime_cur = st_cur.st_mtime;
+  data->mtime_cur = st_cur.st_mtime;
   ctx->mtime = st.st_mtime;
 
   memset (&mhs, 0, sizeof (mhs));
