@@ -143,7 +143,8 @@ typedef enum
   M_FORMAT_OPTIONAL	= (1<<3),
   M_FORMAT_STAT_FILE	= (1<<4), /* used by mutt_attach_fmt */
   M_FORMAT_ARROWCURSOR	= (1<<5), /* reserve space for arrow_cursor */
-  M_FORMAT_INDEX	= (1<<6)  /* this is a main index entry */
+  M_FORMAT_INDEX	= (1<<6), /* this is a main index entry */
+  M_FORMAT_NOFILTER	= (1<<7)  /* do not allow filtering on this pass */
 } format_flag;
 
 /* types for mutt_add_hook() */
@@ -338,6 +339,7 @@ enum
   OPTBEEPNEW,
   OPTBOUNCEDELIVERED,
   OPTBRAILLEFRIENDLY,
+  OPTCHECKMBOXSIZE,
   OPTCHECKNEW,
   OPTCOLLAPSEUNREAD,
   OPTCONFIRMAPPEND,
@@ -370,6 +372,7 @@ enum
   OPTHIDETHREADSUBJECT,
   OPTHIDETOPLIMITED,
   OPTHIDETOPMISSING,
+  OPTIGNORELWS,
   OPTIGNORELISTREPLYTO,
 #ifdef USE_IMAP
   OPTIMAPCHECKSUBSCRIBED,
@@ -453,6 +456,7 @@ enum
   OPTXMAILER,
 
   OPTCRYPTUSEGPGME,
+  OPTCRYPTUSEPKA,
 
   /* PGP options */
   
@@ -641,6 +645,7 @@ typedef struct body
 				 * If NULL, filename is used 
 				 * instead.
 				 */
+  char *charset;                /* charset of attached file */
   CONTENT *content;             /* structure used to store detailed info about
 				 * the content of the attachment.  this is used
 				 * to determine what content-transfer-encoding
@@ -843,18 +848,19 @@ enum
   M_ACL_INSERT,
   M_ACL_POST,
   M_ACL_CREATE,
+  M_ACL_DELMX,
   M_ACL_DELETE,
+  M_ACL_EXPUNGE,
   M_ACL_ADMIN,
 
   RIGHTSMAX
 };
 
-typedef struct
+typedef struct _context
 {
   char *path;
   FILE *fp;
   time_t mtime;
-  time_t mtime_cur;		/* used with maildir folders */
   off_t size;
   off_t vsize;
   char *pattern;                /* limit pattern string */
@@ -875,9 +881,6 @@ typedef struct
   int deleted;			/* how many deleted messages */
   int flagged;			/* how many flagged messages */
   int msgnotreadyet;		/* which msg "new" in pager, -1 if none */
-#if defined USE_POP || defined USE_IMAP
-  void *data;			/* driver specific data */
-#endif /* USE_IMAP */
 
   short magic;			/* mailbox type */
 
@@ -891,6 +894,10 @@ typedef struct
   unsigned int quiet : 1;	/* inhibit status messages? */
   unsigned int collapsed : 1;   /* are all threads collapsed? */
   unsigned int closing : 1;	/* mailbox is being closed */
+
+  /* driver hooks */
+  void *data;			/* driver specific data */
+  int (*mx_close)(struct _context *);
 } CONTEXT;
 
 typedef struct
