@@ -442,7 +442,7 @@ int mutt_write_mime_body (BODY *a, FILE *f)
   }
 
   if (a->type == TYPETEXT && (!a->noconv))
-    fc = fgetconv_open (fpin, Charset, 
+    fc = fgetconv_open (fpin, a->charset, 
 			mutt_get_body_charset (send_charset, sizeof (send_charset), a),
 			0);
   else
@@ -632,7 +632,7 @@ static size_t convert_file_to (FILE *file, const char *fromcode,
   CONTENT_STATE *states;
   size_t *score;
 
-  cd1 = mutt_iconv_open ("UTF-8", fromcode, 0);
+  cd1 = mutt_iconv_open ("utf-8", fromcode, 0);
   if (cd1 == (iconv_t)(-1))
     return -1;
 
@@ -642,8 +642,8 @@ static size_t convert_file_to (FILE *file, const char *fromcode,
   infos  = safe_calloc (ncodes, sizeof (CONTENT));
 
   for (i = 0; i < ncodes; i++)
-    if (ascii_strcasecmp (tocodes[i], "UTF-8"))
-      cd[i] = mutt_iconv_open (tocodes[i], "UTF-8", 0);
+    if (ascii_strcasecmp (tocodes[i], "utf-8"))
+      cd[i] = mutt_iconv_open (tocodes[i], "utf-8", 0);
     else
       /* Special case for conversion to UTF-8 */
       cd[i] = (iconv_t)(-1), score[i] = (size_t)(-1);
@@ -842,6 +842,7 @@ CONTENT *mutt_get_content_info (const char *fname, BODY *b)
   CONTENT *info;
   CONTENT_STATE state;
   FILE *fp = NULL;
+  char *fromcode = NULL;
   char *tocode;
   char buffer[100];
   char chsbuf[STRING];
@@ -876,15 +877,18 @@ CONTENT *mutt_get_content_info (const char *fname, BODY *b)
   if (b != NULL && b->type == TYPETEXT && (!b->noconv && !b->force_charset))
   {
     char *chs = mutt_get_parameter ("charset", b->parameter);
+    char *fchs = b->use_disp ? ((AttachCharset && *AttachCharset) ?
+                                AttachCharset : Charset) : Charset;
     if (Charset && (chs || SendCharset) &&
-	convert_file_from_to (fp, Charset, chs ? chs : SendCharset,
-			      0, &tocode, info) != (size_t)(-1))
+        convert_file_from_to (fp, fchs, chs ? chs : SendCharset,
+                              &fromcode, &tocode, info) != (size_t)(-1))
     {
       if (!chs)
       {
 	mutt_canonical_charset (chsbuf, sizeof (chsbuf), tocode);
 	mutt_set_parameter ("charset", chsbuf, &b->parameter);
       }
+      b->charset = fromcode;
       FREE (&tocode);
       safe_fclose (&fp);
       return info;
@@ -1267,6 +1271,7 @@ BODY *mutt_make_message_attach (CONTEXT *ctx, HEADER *hdr, int attach_msg)
   body->unlink = 1;
   body->use_disp = 0;
   body->disposition = DISPINLINE;
+  body->noconv = 1;
 
   mutt_parse_mime_message (ctx, hdr);
 
@@ -1399,6 +1404,23 @@ static int get_toplevel_encoding (BODY *a)
   return (e);
 }
 
+/* check for duplicate boundary. return 1 if duplicate */
+static int mutt_check_boundary (const char* boundary, BODY *b)
+{
+  char* p;
+
+  if (b->parts && mutt_check_boundary (boundary, b->parts))
+    return 1;
+
+  if (b->next && mutt_check_boundary (boundary, b->next))
+    return 1;
+
+  if ((p = mutt_get_parameter ("boundary", b->parameter))
+      && !ascii_strcmp (p, boundary))
+    return 1;
+  return 0;
+}
+
 BODY *mutt_make_multipart (BODY *b)
 {
   BODY *new;
@@ -1407,7 +1429,14 @@ BODY *mutt_make_multipart (BODY *b)
   new->type = TYPEMULTIPART;
   new->subtype = safe_strdup ("mixed");
   new->encoding = get_toplevel_encoding (b);
-  mutt_generate_boundary (&new->parameter);
+  do
+  {
+    mutt_generate_boundary (&new->parameter);
+    if (mutt_check_boundary (mutt_get_parameter ("boundary", new->parameter),
+			     b))
+      mutt_delete_parameter ("boundary", &new->parameter);
+  }
+  while (!mutt_get_parameter ("boundary", new->parameter));
   new->use_disp = 0;  
   new->disposition = DISPINLINE;
   new->parts = b;
@@ -1726,6 +1755,13 @@ int mutt_write_rfc822_header (FILE *fp, ENVELOPE *env, BODY *attach,
     fprintf (fp, "From: %s\n", buffer);
   }
 
+  if (env->sender && !privacy)
+  {
+    buffer[0] = 0;
+    rfc822_write_address (buffer, sizeof (buffer), env->sender, 0);
+    fprintf (fp, "Sender: %s\n", buffer);
+  }
+
   if (env->to)
   {
     fputs ("To: ", fp);
@@ -1920,7 +1956,8 @@ static RETSIGTYPE alarm_handler (int sig)
    msg (in)		temp file containing message to send
    tempfile (out)	if sendmail is put in the background, this points
    			to the temporary file containing the stdout of the
-			child process */
+			child process. If it is NULL, stderr and stdout
+                        are not redirected. */
 static int
 send_msg (const char *path, char **args, const char *msg, char **tempfile)
 {
@@ -1935,7 +1972,7 @@ send_msg (const char *path, char **args, const char *msg, char **tempfile)
   sigaddset (&set, SIGTSTP);
   sigprocmask (SIG_BLOCK, &set, NULL);
 
-  if (SendmailWait >= 0)
+  if (SendmailWait >= 0 && tempfile)
   {
     char tmp[_POSIX_PATH_MAX];
 
@@ -1956,16 +1993,19 @@ send_msg (const char *path, char **args, const char *msg, char **tempfile)
     setsid ();
   
     /* next we close all open files */
+    close (0);
 #if defined(OPEN_MAX)
-    for (fd = 0; fd < OPEN_MAX; fd++)
+    for (fd = tempfile ? 1 : 3; fd < OPEN_MAX; fd++)
       close (fd);
 #elif defined(_POSIX_OPEN_MAX)
-    for (fd = 0; fd < _POSIX_OPEN_MAX; fd++)
+    for (fd = tempfile ? 1 : 3; fd < _POSIX_OPEN_MAX; fd++)
       close (fd);
 #else
-    close (0);
-    close (1);
-    close (2);
+    if (tempfile)
+    {
+      close (1);
+      close (2);
+    }
 #endif
 
     /* now the second fork() */
@@ -1979,7 +2019,7 @@ send_msg (const char *path, char **args, const char *msg, char **tempfile)
       }
       unlink (msg);
 
-      if (SendmailWait >= 0)
+      if (SendmailWait >= 0 && tempfile)
       {
 	/* *tempfile will be opened as stdout */
 	if (open (*tempfile, O_WRONLY | O_APPEND | O_CREAT | O_EXCL, 0600) < 0)
@@ -1988,7 +2028,7 @@ send_msg (const char *path, char **args, const char *msg, char **tempfile)
 	if (dup (1) < 0)
 	  _exit (S_ERR);
       }
-      else 
+      else if (tempfile)
       {
 	if (open ("/dev/null", O_WRONLY | O_APPEND) < 0)	/* stdout */
 	  _exit (S_ERR);
@@ -1996,13 +2036,14 @@ send_msg (const char *path, char **args, const char *msg, char **tempfile)
 	  _exit (S_ERR);
       }
 
-      execv (path, args);
+      execvp (path, args);
       _exit (S_ERR);
     }
     else if (pid == -1)
     {
       unlink (msg);
-      FREE (tempfile);		/* __FREE_CHECKED__ */
+      if (tempfile)
+	FREE (tempfile);		/* __FREE_CHECKED__ */
       _exit (S_ERR);
     }
 
@@ -2030,7 +2071,7 @@ send_msg (const char *path, char **args, const char *msg, char **tempfile)
     if (waitpid (pid, &st, 0) > 0)
     {
       st = WIFEXITED (st) ? WEXITSTATUS (st) : S_ERR;
-      if (SendmailWait && st == (0xff & EX_OK))
+      if (SendmailWait && st == (0xff & EX_OK) && tempfile)
       {
 	unlink (*tempfile); /* no longer needed */
 	FREE (tempfile);		/* __FREE_CHECKED__ */
@@ -2040,7 +2081,7 @@ send_msg (const char *path, char **args, const char *msg, char **tempfile)
     {
       st = (SendmailWait > 0 && errno == EINTR && SigAlrm) ?
 	      S_BKG : S_ERR;
-      if (SendmailWait > 0)
+      if (SendmailWait > 0 && tempfile)
       {
 	unlink (*tempfile);
 	FREE (tempfile);		/* __FREE_CHECKED__ */
@@ -2051,7 +2092,7 @@ send_msg (const char *path, char **args, const char *msg, char **tempfile)
     alarm (0);
     sigaction (SIGALRM, &oldalrm, NULL);
 
-    if (kill (ppid, 0) == -1 && errno == ESRCH)
+    if (kill (ppid, 0) == -1 && errno == ESRCH && tempfile)
     {
       /* the parent is already dead */
       unlink (*tempfile);
@@ -2169,7 +2210,7 @@ mutt_invoke_sendmail (ADDRESS *from,	/* the sender */
   
   args[argslen++] = NULL;
 
-  if ((i = send_msg (path, args, msg, &childout)) != (EX_OK & 0xff))
+  if ((i = send_msg (path, args, msg, option(OPTNOCURSES) ? NULL : &childout)) != (EX_OK & 0xff))
   {
     if (i != S_BKG)
     {
@@ -2361,6 +2402,12 @@ static int _mutt_bounce_message (FILE *fp, HEADER *h, ADDRESS *to, const char *r
     mutt_copy_bytes (fp, f, h->content->length);
     fclose (f);
 
+#if USE_SMTP
+    if (SmtpUrl)
+      ret = mutt_smtp_send (env_from, to, NULL, NULL, tempfile,
+                            h->content->encoding == ENC8BIT);
+    else
+#endif /* USE_SMTP */
     ret = mutt_invoke_sendmail (env_from, to, NULL, NULL, tempfile,
 			  	h->content->encoding == ENC8BIT);
   }
@@ -2611,7 +2658,7 @@ int mutt_write_fcc (const char *path, HEADER *hdr, const char *msgid, int post,
     rewind (tempfp);
     while (fgets (sasha, sizeof (sasha), tempfp) != NULL)
       lines++;
-    fprintf (msg->fp, "Content-Length: " OFF_T_FMT "\n", ftello (tempfp));
+    fprintf (msg->fp, "Content-Length: " OFF_T_FMT "\n", (LOFF_T) ftell (tempfp));
     fprintf (msg->fp, "Lines: %d\n\n", lines);
 
     /* copy the body and clean up */
