@@ -37,10 +37,12 @@ static struct mapping_t UrlMap[] =
   { "file", 	U_FILE },
   { "imap", 	U_IMAP },
   { "imaps", 	U_IMAPS },
-  { "pop",  	U_POP  },
-  { "pops", 	U_POPS  },
+  { "pop",  	U_POP },
+  { "pops", 	U_POPS },
   { "mailto",	U_MAILTO },
-  { NULL,	U_UNKNOWN}
+  { "smtp",     U_SMTP },
+  { "smtps",    U_SMTPS },
+  { NULL,	U_UNKNOWN }
 };
 
 
@@ -182,7 +184,8 @@ int url_ciss_tostring (ciss_url_t* ciss, char* dest, size_t len, int flags)
 
   if (ciss->host)
   {
-    safe_strcat (dest, len, "//");
+    if (!(flags & U_PATH))
+      safe_strcat (dest, len, "//");
     len -= (l = strlen (dest)); dest += l;
     
     if (ciss->user) {
